/*-
 * Copyright (c) 1991 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by the University of
 *      California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      @(#)if_nereg.h  7.1 (Berkeley) 5/9/91
 */

/*
 * NE1000/2000 Ethernet Card registers
 */

/* This card uses a DS8390 Ethernet controller in at the beginning of
   its i/o space */

#define ne_data         0x10    /* Data Transfer port */
#define ne_reset        0x1F    /* Card Reset port */

#define PKTSZ		6	/* Size of transmit buffer */

/* Span of memory on an NE2000 */
#define TBUF16		0x40	/* Starting location of Transmit Buffer */
#define RBUFEND16	0x80	/* Ending location of Receive Buffer */

/* Span of memory on an NE1000 */
#define TBUF8		0x20	/* Starting location of Transmit Buffer */
#define RBUFEND8	0x40	/* Ending location of Receive Buffer */

#if 0
#define PKTSZ   3*512           /* Size of transmit buffer */

/* Span of memory on an NE2000 */
#define TBUF16  (16*1024)       /* Starting location of Transmit Buffer */
#define RBUFEND16       (32*1024)       /* Ending location of Receive Buffer */

/* Span of memory on an NE1000 */
#define TBUF8   (8*1024)        /* Starting location of Transmit Buffer */
#define RBUFEND8        (16*1024)       /* Ending location of Receive Buffer */
#endif
