/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /gd4/gnu/cvsroot/gnumach/mig/lexxer.c,v 1.1 1997/04/14 20:28:00 thomas Exp $
 */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#include <stdio.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>
#include <unistd.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef __TURBOC__
 #pragma warn -rch
 #pragma warn -use
#include <io.h>
#include <stdlib.h>
#define YY_USE_CONST
#define YY_USE_PROTOS
#endif

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int yy_size_t;


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart YY_PROTO(( FILE *input_file ));

void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void yy_load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));
void yy_flush_buffer YY_PROTO(( YY_BUFFER_STATE b ));
#define YY_FLUSH_BUFFER yy_flush_buffer( yy_current_buffer )

YY_BUFFER_STATE yy_scan_buffer YY_PROTO(( char *base, yy_size_t size ));
YY_BUFFER_STATE yy_scan_string YY_PROTO(( yyconst char *str ));
YY_BUFFER_STATE yy_scan_bytes YY_PROTO(( yyconst char *bytes, int len ));

static void *yy_flex_alloc YY_PROTO(( yy_size_t ));
static void *yy_flex_realloc YY_PROTO(( void *, yy_size_t ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (yy_current_buffer->yy_at_bol)

typedef unsigned char YY_CHAR;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
typedef int yy_state_type;
extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state YY_PROTO(( void ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state ));
static int yy_get_next_buffer YY_PROTO(( void ));
static void yy_fatal_error YY_PROTO(( yyconst char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 100
#define YY_END_OF_BUFFER 101
static yyconst short int yy_accept[515] =
    {   0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,  101,   99,   97,   98,   94,   77,   78,   75,
       73,   72,   74,   76,   88,   70,   71,   82,   79,   83,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   84,   85,   80,
       87,   86,   81,   89,   99,   99,   99,   96,   96,   95,
       94,    0,    0,   93,   88,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   11,   87,   87,   87,
       87,   87,   19,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,

       89,    0,   90,    0,    0,   91,    0,    0,    0,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   12,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,    0,   92,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   31,
       87,   87,   87,   28,   87,   87,   87,   87,   18,   87,
       87,   36,   87,   87,   87,   87,   20,   87,   87,   13,

       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   24,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   25,   33,   40,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   32,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   42,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   34,   87,   87,   87,   87,    1,
       87,   87,   87,   87,   87,   27,   87,   87,   26,   87,
       87,   87,   87,   87,   87,   39,   87,    2,   87,   87,

       87,   87,   87,    8,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,    9,
       87,   87,   37,   87,   41,   87,   87,   87,    7,   87,
       87,   87,    3,   15,   87,   87,   87,   87,   87,   87,
       87,    6,   87,   87,   45,   87,   35,   87,   30,   87,
       43,   10,   87,   87,   44,   87,   87,   87,   87,   17,
       16,   22,   38,   87,   87,   46,   14,   23,   87,   87,
       87,   29,   87,   21,   87,   87,   87,   87,    5,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,    4,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,

       87,   87,   48,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   54,   53,   87,   87,
       87,   87,   87,   87,   56,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   57,   87,   49,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   87,   87,   58,
       87,   60,   87,   87,   87,   55,   61,   87,   62,   87,
       65,   87,   67,   87,   50,   51,   52,   87,   87,   87,
       87,   87,   87,   87,   87,   87,   87,   69,   87,   87,

       87,   87,   59,   87,   66,   87,   47,   87,   87,   87,
       63,   64,   68,    0
    } ;

static yyconst int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    4,    1,    5,    6,    7,    1,    1,    1,    8,
        9,   10,   11,   12,   13,    7,   14,   15,   16,   17,
       18,   19,   15,   20,   15,   21,   15,   22,   23,   24,
       25,   26,    1,    1,   27,   28,   29,   30,   31,   32,
       33,   34,   35,   36,   37,   38,   39,   40,   41,   42,
       43,   44,   45,   46,   47,   48,   49,   50,   51,   36,
       52,    1,   53,   54,   55,    1,   56,   57,   58,   59,

       60,   32,   61,   62,   63,   36,   64,   65,   66,   67,
       68,   69,   43,   70,   71,   72,   73,   74,   49,   50,
       75,   36,    1,   76,    1,   77,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst int yy_meta[78] =
    {   0,
        1,    1,    2,    1,    1,    1,    3,    1,    1,    1,
        1,    1,    3,    3,    4,    4,    4,    4,    4,    4,
        4,    1,    1,    1,    1,    1,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    1,    1,    1,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    1,    1
    } ;

static yyconst short int yy_base[524] =
    {   0,
        0,    3,    9,  979,   86,  163,   97,  101,  111,  124,
      115,  129,  984,  986,  986,  986,  132,  986,  986,  986,
      986,  986,  986,  986,  139,  986,  986,  986,  986,  986,
       76,    0,  135,   62,  100,  131,  152,  152,  134,  143,
      162,  152,  169,  210,  126,  189,  172,  986,  986,  986,
      150,  986,  986,    0,  978,  956,  976,  986,  986,  986,
      246,  273,  282,  302,  293,    0,  181,  185,  171,  163,
      197,  198,  260,  202,  186,  202,  283,  287,  211,  951,
      282,  284,    0,  286,  211,  285,  289,  293,  290,  294,
      300,  305,  297,  297,  314,  303,  307,  316,  317,  315,

        0,  974,  986,  952,  972,  986,  377,  381,  971,  332,
      334,  300,  333,  330,  319,  342,  338,  346,  360,  358,
      356,  360,  359,  364,  366,  941,  367,  381,  386,  369,
      368,  387,  374,  379,  373,  375,  374,  381,  382,  383,
      381,  386,  403,  397,  407,  392,  407,  969,  986,  405,
      409,  410,  428,  416,  417,  426,  423,  424,  423,  426,
      425,  445,  425,  427,  443,  918,  433,  446,  447,  447,
      439,  446,  458,  460,  447,  469,  469,  474,  470,    0,
      475,  485,  464,    0,  476,  482,  479,  470,    0,  488,
      493,    0,  480,  492,  489,  481,    0,  496,  490,    0,

      493,  504,  503,  507,  933,  503,  928,  523,  510,  531,
      505,  531,    0,  522,  518,  526,  525,  536,  527,  519,
      528,  531,  533,  529,  537,  543,  543,  541,  533,  546,
      559,  563,  557,    0,    0,    0,  558,  567,  925,  566,
      565,  571,  569,  576,  574,  572,  579,  576,  592,  604,
      586,  578,  585,    0,  585,  589,  595,  608,  603,  608,
      605,  611,  625,    0,  613,  620,  625,  635,  622,  936,
      629,  630,  634,  639,    0,  636,  631,  637,  637,    0,
      641,  652,  640,  644,  648,    0,  649,  660,    0,  653,
      936,  665,  653,  652,  673,    0,  667,    0,  678,  671,

      682,  912,  678,    0,  678,  683,  689,  693,  682,  688,
      688,  692,  701,  695,  687,  694,  701,  699,  702,    0,
      698,  705,    0,  706,    0,  704,  711,  920,    0,  719,
      728,  727,    0,    0,  722,  720,  721,  933,  743,  727,
      733,    0,  734,  914,    0,  744,    0,  751,    0,  912,
        0,    0,  754,  738,    0,  912,  750,  755,  754,    0,
        0,    0,    0,  749,  919,    0,    0,    0,  910,  757,
      751,    0,  928,    0,  749,  766,  903,  754,    0,  792,
      769,  244,  768,  917,   96,  915,  924,  908,  913,    0,
      906,  910,  904,  922,  906,  901,  909,  897,  192,  917,

      899,  897,    0,  903,  909,  895,  887,  906,  905,  904,
      883,  887,  894,  896,  884,  898,    0,    0,  873,  894,
      871,  870,  885,  868,    0,  882,  877,  893,  874,  887,
      872,   97,  875,  763,  882,  867,  873,  881,  867,  879,
      878,  877,  863,  879,  874,  873,  848,  873,    0,  861,
      845,  859,  869,  857,  854,  856,  865,  853,  863,  845,
      860,  812,  859,  857,  857,  852,  854,  853,  853,    0,
      835,    0,  861,  863,  860,    0,  823,  842,  821,  840,
        0,  839,  818,  828,    0,    0,    0,  830,  822,  822,
      830,  807,  797,  804,  785,  792,  774,    0,  494,  329,

      319,  267,    0,  249,    0,  184,    0,  114,  112,   93,
        0,    0,    0,  986,  839,  843,  847,    4,  849,  853,
      857,  861,  865
    } ;

static yyconst short int yy_def[524] =
    {   0,
      515,  515,  514,    3,  516,  516,  515,  515,  515,  515,
      517,  517,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  514,  514,  514,
      518,  514,  514,  519,  520,  521,  522,  514,  514,  514,
      514,  514,  514,  514,  514,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,

      519,  520,  514,  521,  522,  514,  514,  514,  523,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  523,  514,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,

      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,

      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,

      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,

      518,  518,  518,  518,  518,  518,  518,  518,  518,  518,
      518,  518,  518,    0,  514,  514,  514,  514,  514,  514,
      514,  514,  514
    } ;

static yyconst short int yy_nxt[1064] =
    {   0,
      514,   15,   16,   15,   15,   16,   15,   66,   17,   14,
       15,   16,   15,   14,   14,   14,   18,   19,   20,   21,
       22,   23,   24,   25,   25,   25,   25,   25,   25,   25,
       26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
       36,   32,   32,   37,   32,   38,   32,   39,   40,   41,
       42,   32,   43,   44,   45,   46,   32,   47,   32,   32,
       48,   49,   50,   32,   31,   32,   33,   34,   35,   32,
       32,   37,   38,   32,   51,   40,   41,   42,   43,   44,
       45,   46,   32,   32,   52,   53,   14,   15,   16,   15,
       14,   14,   73,   14,   14,   14,   14,   14,   15,   16,

       15,   55,   15,   16,   15,   55,   17,   14,   14,   14,
       14,   14,   15,   16,   15,   57,   59,   60,   59,   67,
       56,   73,  397,  513,   56,   15,   16,   15,   57,   17,
       59,   60,   59,   62,   61,   63,  398,   14,   14,   14,
      441,  442,  512,   74,  511,   67,   64,   64,   64,   64,
       64,   64,   64,   65,   65,   65,   65,   65,   65,   65,
       80,   14,   14,   14,   15,   16,   15,   14,   17,   74,
       14,   14,   14,   14,   14,   68,   96,   75,   81,   69,
       70,   71,   79,   82,   14,   14,   14,   14,   14,   72,
       76,   77,   85,   83,   81,   86,   78,   87,  100,   88,

       96,  112,   68,   75,   81,   69,   70,   71,   84,   89,
       82,   79,  510,  113,   14,   14,   14,   76,   77,   85,
       81,   86,   78,   97,  110,  119,   87,  100,   88,  411,
      112,  111,   98,   99,   84,  412,   89,  113,   14,   14,
       90,  114,  115,  120,   91,  118,   92,   62,  131,   63,
      110,   97,  119,   93,  125,   94,   95,  111,   98,   99,
       64,   64,   64,   64,   64,   64,   64,  114,  115,   90,
      120,  118,   91,   92,   62,  131,   62,  509,  391,   93,
      125,   94,   95,   62,  392,   63,  116,  107,  107,  107,
      107,  107,  107,  107,  393,  508,   64,   64,   64,   64,

       64,   64,   64,  108,  117,  108,  109,   65,   65,   65,
       65,   65,   65,   65,  127,  116,   64,   64,   64,   64,
       64,   64,   64,  121,  123,  132,  124,  140,  122,  128,
      117,  130,  129,  133,  134,  135,  136,  137,  138,  139,
      141,  142,  127,  152,  143,  144,  145,  146,  507,  147,
      121,  123,  132,  124,  122,  128,  140,  130,  150,  133,
      154,  134,  136,  137,  155,  138,  141,  139,  506,  152,
      142,  143,  144,  151,  153,  145,  146,  147,  108,  156,
      108,  109,  108,  157,  108,  109,  158,  150,  159,  154,
      155,  107,  107,  107,  107,  107,  107,  107,  160,  163,

      151,  153,  161,  162,  164,  165,  156,  167,  174,  157,
      169,  168,  170,  158,  171,  173,  175,  159,  172,  176,
      177,  178,  179,  180,  181,  160,  163,  182,  161,  162,
      183,  164,  165,  184,  167,  187,  174,  168,  185,  169,
      171,  170,  172,  175,  173,  176,  177,  178,  186,  179,
      180,  181,  188,  182,  190,  189,  183,  191,  192,  193,
      194,  187,  184,  195,  202,  185,  196,  197,  198,  199,
      200,  201,  203,  204,  206,  186,  207,  208,  188,  189,
      190,  209,  210,  191,  211,  193,  194,  192,  212,  213,
      195,  202,  196,  197,  198,  199,  200,  214,  203,  215,

      201,  206,  204,  216,  217,  207,  208,  218,  210,  209,
      219,  211,  220,  221,  222,  226,  223,  212,  213,  224,
      225,  214,  227,  228,  505,  229,  230,  232,  215,  231,
      233,  216,  235,  217,  218,  234,  236,  219,  222,  220,
      237,  226,  221,  223,  238,  245,  224,  225,  240,  242,
      227,  229,  228,  232,  230,  243,  231,  244,  233,  235,
      246,  234,  248,  247,  236,  249,  251,  237,  250,  253,
      252,  238,  245,  254,  240,  255,  256,  259,  242,  257,
      258,  243,  260,  262,  261,  263,  244,  264,  248,  246,
      247,  265,  249,  253,  250,  251,  252,  266,  267,  254,

      271,  255,  256,  257,  272,  259,  258,  262,  273,  260,
      261,  268,  263,  269,  274,  275,  264,  276,  277,  278,
      265,  279,  280,  286,  266,  267,  287,  284,  271,  285,
      288,  272,  281,  282,  289,  273,  290,  268,  291,  269,
      274,  292,  275,  276,  277,  283,  278,  279,  293,  286,
      294,  280,  295,  287,  284,  285,  288,  296,  297,  298,
      289,  281,  282,  290,  299,  300,  301,  291,  292,  303,
      304,  305,  283,  306,  308,  293,  294,  307,  310,  295,
      309,  311,  312,  313,  297,  296,  298,  314,  315,  316,
      317,  299,  301,  318,  300,  320,  303,  304,  305,  321,

      308,  306,  322,  323,  307,  310,  309,  324,  311,  313,
      325,  312,  327,  314,  326,  315,  316,  329,  330,  317,
      318,  331,  332,  333,  320,  321,  322,  334,  335,  336,
      337,  338,  323,  340,  324,  339,  344,  341,  325,  342,
      326,  327,  343,  345,  329,  330,  346,  351,  331,  347,
      332,  348,  333,  334,  349,  335,  336,  337,  352,  340,
      338,  353,  339,  341,  344,  354,  342,  356,  343,  345,
      355,  358,  359,  346,  363,  347,  351,  348,  360,  361,
      349,  364,  366,  367,  369,  370,  375,  352,  371,  353,
      376,  354,  372,  356,  355,  378,  379,  381,  359,  390,

      358,  394,  444,  363,  360,  361,  445,  446,  395,  367,
      364,  366,  369,  504,  370,  375,  371,  376,  372,  382,
      383,  378,  503,  381,  502,  379,  384,  473,  390,  474,
      385,  475,  476,  386,  501,  387,  388,  500,  389,   14,
       14,   14,   14,   54,   54,   54,   54,   58,   58,   58,
       58,  101,  101,  102,  499,  102,  102,  104,  498,  104,
      104,  105,  497,  105,  105,  148,  148,  148,  148,  496,
      495,  494,  493,  492,  491,  490,  489,  488,  487,  486,
      485,  484,  483,  482,  481,  480,  479,  478,  477,  472,
      471,  470,  469,  468,  467,  466,  465,  464,  463,  462,

      461,  460,  459,  458,  457,  456,  455,  454,  453,  452,
      451,  450,  449,  448,  447,  443,  440,  439,  438,  437,
      436,  435,  434,  433,  432,  431,  430,  429,  428,  427,
      426,  425,  424,  423,  422,  421,  420,  419,  418,  417,
      416,  415,  414,  413,  410,  409,  408,  407,  406,  405,
      404,  403,  402,  401,  400,  399,  396,  380,  377,  374,
      373,  368,  365,  362,  357,  350,  328,  319,  302,  270,
      241,  239,  205,  149,  166,  149,  106,  103,  103,  126,
      106,  103,  103,  514,   17,   13,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,

      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514
    } ;

static yyconst short int yy_chk[1064] =
    {   0,
        0,    1,    1,    1,    2,    2,    2,  518,    2,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    5,    5,    5,    5,
        5,    5,   34,    5,    5,    5,    5,    5,    7,    7,

        7,    7,    8,    8,    8,    8,    8,    5,    5,    5,
        5,    5,    9,    9,    9,    9,   11,   11,   11,   31,
        7,   34,  385,  510,    8,   10,   10,   10,   10,   10,
       12,   12,   12,   17,   12,   17,  385,    5,    5,    5,
      432,  432,  509,   35,  508,   31,   17,   17,   17,   17,
       17,   17,   17,   25,   25,   25,   25,   25,   25,   25,
       39,    5,    5,    6,    6,    6,    6,    6,    6,   35,
        6,    6,    6,    6,    6,   33,   45,   36,   39,   33,
       33,   33,   38,   40,    6,    6,    6,    6,    6,   33,
       37,   37,   42,   41,   51,   42,   37,   43,   47,   43,

       45,   69,   33,   36,   39,   33,   33,   33,   41,   43,
       40,   38,  506,   70,    6,    6,    6,   37,   37,   42,
       51,   42,   37,   46,   67,   75,   43,   47,   43,  399,
       69,   68,   46,   46,   41,  399,   43,   70,    6,    6,
       44,   71,   72,   76,   44,   74,   44,   61,   85,   61,
       67,   46,   75,   44,   79,   44,   44,   68,   46,   46,
       61,   61,   61,   61,   61,   61,   61,   71,   72,   44,
       76,   74,   44,   44,   62,   85,   62,  504,  382,   44,
       79,   44,   44,   63,  382,   63,   73,   62,   62,   62,
       62,   62,   62,   62,  382,  502,   63,   63,   63,   63,

       63,   63,   63,   64,   73,   64,   64,   65,   65,   65,
       65,   65,   65,   65,   81,   73,   64,   64,   64,   64,
       64,   64,   64,   77,   78,   86,   78,   93,   77,   82,
       73,   84,   82,   87,   88,   88,   89,   90,   91,   92,
       94,   95,   81,  112,   96,   97,   98,   99,  501,  100,
       77,   78,   86,   78,   77,   82,   93,   84,  110,   87,
      114,   88,   89,   90,  115,   91,   94,   92,  500,  112,
       95,   96,   97,  111,  113,   98,   99,  100,  107,  116,
      107,  107,  108,  117,  108,  108,  118,  110,  119,  114,
      115,  107,  107,  107,  107,  107,  107,  107,  120,  123,

      111,  113,  121,  122,  124,  125,  116,  127,  133,  117,
      128,  127,  129,  118,  130,  132,  134,  119,  131,  135,
      136,  137,  138,  139,  140,  120,  123,  141,  121,  122,
      142,  124,  125,  143,  127,  146,  133,  127,  144,  128,
      130,  129,  131,  134,  132,  135,  136,  137,  145,  138,
      139,  140,  147,  141,  151,  150,  142,  152,  153,  154,
      155,  146,  143,  156,  163,  144,  157,  158,  159,  160,
      161,  162,  164,  165,  167,  145,  168,  169,  147,  150,
      151,  170,  171,  152,  172,  154,  155,  153,  173,  174,
      156,  163,  157,  158,  159,  160,  161,  175,  164,  176,

      162,  167,  165,  177,  178,  168,  169,  179,  171,  170,
      179,  172,  181,  182,  183,  188,  185,  173,  174,  186,
      187,  175,  190,  191,  499,  193,  194,  196,  176,  195,
      198,  177,  201,  178,  179,  199,  202,  179,  183,  181,
      203,  188,  182,  185,  204,  211,  186,  187,  206,  208,
      190,  193,  191,  196,  194,  209,  195,  210,  198,  201,
      212,  199,  215,  214,  202,  216,  218,  203,  217,  220,
      219,  204,  211,  221,  206,  222,  223,  226,  208,  224,
      225,  209,  227,  229,  228,  230,  210,  231,  215,  212,
      214,  232,  216,  220,  217,  218,  219,  233,  237,  221,

      240,  222,  223,  224,  241,  226,  225,  229,  242,  227,
      228,  238,  230,  238,  243,  244,  231,  245,  246,  247,
      232,  248,  249,  252,  233,  237,  253,  251,  240,  251,
      255,  241,  250,  250,  256,  242,  257,  238,  258,  238,
      243,  259,  244,  245,  246,  250,  247,  248,  260,  252,
      261,  249,  262,  253,  251,  251,  255,  263,  265,  266,
      256,  250,  250,  257,  267,  268,  269,  258,  259,  271,
      272,  273,  250,  274,  277,  260,  261,  276,  279,  262,
      278,  281,  282,  283,  265,  263,  266,  284,  285,  287,
      288,  267,  269,  290,  268,  292,  271,  272,  273,  293,

      277,  274,  294,  295,  276,  279,  278,  297,  281,  283,
      299,  282,  301,  284,  300,  285,  287,  303,  305,  288,
      290,  306,  307,  308,  292,  293,  294,  309,  310,  311,
      312,  313,  295,  315,  297,  314,  319,  316,  299,  317,
      300,  301,  318,  321,  303,  305,  322,  330,  306,  324,
      307,  326,  308,  309,  327,  310,  311,  312,  331,  315,
      313,  332,  314,  316,  319,  335,  317,  337,  318,  321,
      336,  339,  340,  322,  346,  324,  330,  326,  341,  343,
      327,  348,  353,  354,  357,  358,  370,  331,  359,  332,
      371,  335,  364,  337,  336,  375,  376,  378,  340,  381,

      339,  383,  434,  346,  341,  343,  434,  434,  383,  354,
      348,  353,  357,  497,  358,  370,  359,  371,  364,  380,
      380,  375,  496,  378,  495,  376,  380,  462,  381,  462,
      380,  462,  462,  380,  494,  380,  380,  493,  380,  515,
      515,  515,  515,  516,  516,  516,  516,  517,  517,  517,
      517,  519,  519,  520,  492,  520,  520,  521,  491,  521,
      521,  522,  490,  522,  522,  523,  523,  523,  523,  489,
      488,  484,  483,  482,  480,  479,  478,  477,  475,  474,
      473,  471,  469,  468,  467,  466,  465,  464,  463,  461,
      460,  459,  458,  457,  456,  455,  454,  453,  452,  451,

      450,  448,  447,  446,  445,  444,  443,  442,  441,  440,
      439,  438,  437,  436,  435,  433,  431,  430,  429,  428,
      427,  426,  424,  423,  422,  421,  420,  419,  416,  415,
      414,  413,  412,  411,  410,  409,  408,  407,  406,  405,
      404,  402,  401,  400,  398,  397,  396,  395,  394,  393,
      392,  391,  389,  388,  387,  386,  384,  377,  373,  369,
      365,  356,  350,  344,  338,  328,  302,  291,  270,  239,
      207,  205,  166,  148,  126,  109,  105,  104,  102,   80,
       57,   56,   55,   13,    4,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,

      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514,  514,  514,  514,  514,  514,  514,  514,
      514,  514,  514
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
char *yytext;
#line 1 "../gnumach/mig/lexxer.l"
#define INITIAL 0
#line 15 "../gnumach/mig/lexxer.l"
/* 
 * Mach Operating System
 * Copyright (c) 1991,1990 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */

#include <string.h>

#include "mig_string.h"
#include "type.h"
#include "statement.h"
#include "global.h"
#include "parser.h"
#include "lexxer.h"
#include "cpu.h"

#define	stringize(x)	#x

#ifdef	LDEBUG
#define RETURN(sym)							\
{									\
    printf("yylex: returning '%s' (%d)\n", #sym, (sym));		\
    return (sym);							\
}
#else	LDEBUG
#define RETURN(sym)	return (sym)
#endif	LDEBUG

#define	TPRETURN(intype, outtype, tsize)				\
{									\
    yylval.symtype.innumber = (intype);					\
    yylval.symtype.instr = stringize(intype);				\
    yylval.symtype.outnumber = (outtype);				\
    yylval.symtype.outstr = stringize(outtype);				\
    yylval.symtype.size = (tsize);					\
    RETURN(sySymbolicType);						\
}

#define	TRETURN(type, tsize)	TPRETURN(type,type,tsize)

#define	FRETURN(val)							\
{									\
    yylval.flag = (val);						\
    RETURN(syIPCFlag);							\
}

extern YYSTYPE yylval;		/* added by rm */

int lineno;
char *inname;

#ifdef	YY_START
static int oldYYBegin;
#define SAVE_BEGIN oldYYBegin = YY_START
#define RSTR_BEGIN BEGIN oldYYBegin
#else
static struct yysvf *oldYYBegin;
#define SAVE_BEGIN oldYYBegin = yybgin
#define RSTR_BEGIN yybgin = oldYYBegin;
#endif

static void doSharp(const char *body); /* process body of # directives */
#define Normal 1
#define String 2
#define FileName 3
#define QString 4
#define SkipToEOL 5

#line 859 "lex.yy.c"

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap YY_PROTO(( void ));
#else
extern int yywrap YY_PROTO(( void ));
#endif
#endif

#ifndef YY_NO_UNPUT
static void yyunput YY_PROTO(( int c, char *buf_ptr ));
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, yyconst char *, int ));
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif
#endif

#if YY_STACK_USED
static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
#ifndef YY_NO_PUSH_STATE
static void yy_push_state YY_PROTO(( int new_state ));
#endif
#ifndef YY_NO_POP_STATE
static void yy_pop_state YY_PROTO(( void ));
#endif
#ifndef YY_NO_TOP_STATE
static int yy_top_state YY_PROTO(( void ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yylex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	if ( yyleng > 0 ) \
		yy_current_buffer->yy_at_bol = \
				(yytext[yyleng - 1] == '\n'); \
	YY_USER_ACTION

YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;

#line 101 "../gnumach/mig/lexxer.l"


#line 1012 "lex.yy.c"

	if ( yy_init )
		{
		yy_init = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! yy_current_buffer )
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 515 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 986 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 103 "../gnumach/mig/lexxer.l"
RETURN(syRoutine);
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 104 "../gnumach/mig/lexxer.l"
RETURN(syFunction);
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 105 "../gnumach/mig/lexxer.l"
RETURN(syProcedure);
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 106 "../gnumach/mig/lexxer.l"
RETURN(sySimpleProcedure);
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 107 "../gnumach/mig/lexxer.l"
RETURN(sySimpleRoutine);
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 108 "../gnumach/mig/lexxer.l"
RETURN(sySubsystem);
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 109 "../gnumach/mig/lexxer.l"
RETURN(syMsgOption);
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 110 "../gnumach/mig/lexxer.l"
RETURN(syMsgSeqno);
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 111 "../gnumach/mig/lexxer.l"
RETURN(syWaitTime);
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 112 "../gnumach/mig/lexxer.l"
RETURN(syNoWaitTime);
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 113 "../gnumach/mig/lexxer.l"
RETURN(syIn);
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 114 "../gnumach/mig/lexxer.l"
RETURN(syOut);
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 115 "../gnumach/mig/lexxer.l"
RETURN(syInOut);
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 116 "../gnumach/mig/lexxer.l"
RETURN(syRequestPort);
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 117 "../gnumach/mig/lexxer.l"
RETURN(syReplyPort);
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 118 "../gnumach/mig/lexxer.l"
RETURN(syUReplyPort);
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 119 "../gnumach/mig/lexxer.l"
RETURN(sySReplyPort);
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 120 "../gnumach/mig/lexxer.l"
RETURN(syArray);
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 121 "../gnumach/mig/lexxer.l"
RETURN(syOf);
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 122 "../gnumach/mig/lexxer.l"
RETURN(syErrorProc);
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 123 "../gnumach/mig/lexxer.l"
RETURN(syServerPrefix);
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 124 "../gnumach/mig/lexxer.l"
RETURN(syUserPrefix);
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 125 "../gnumach/mig/lexxer.l"
RETURN(syServerDemux);
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 126 "../gnumach/mig/lexxer.l"
RETURN(syRCSId);
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 127 "../gnumach/mig/lexxer.l"
RETURN(syImport);
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 128 "../gnumach/mig/lexxer.l"
RETURN(syUImport);
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 129 "../gnumach/mig/lexxer.l"
RETURN(sySImport);
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 130 "../gnumach/mig/lexxer.l"
RETURN(syType);
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 131 "../gnumach/mig/lexxer.l"
RETURN(syKernelServer);
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 132 "../gnumach/mig/lexxer.l"
RETURN(syKernelUser);
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 133 "../gnumach/mig/lexxer.l"
RETURN(sySkip);
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 134 "../gnumach/mig/lexxer.l"
RETURN(syStruct);
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 135 "../gnumach/mig/lexxer.l"
RETURN(syInTran);
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 136 "../gnumach/mig/lexxer.l"
RETURN(syOutTran);
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 137 "../gnumach/mig/lexxer.l"
RETURN(syDestructor);
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 138 "../gnumach/mig/lexxer.l"
RETURN(syCType);
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 139 "../gnumach/mig/lexxer.l"
RETURN(syCUserType);
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 140 "../gnumach/mig/lexxer.l"
RETURN(syCServerType);
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 141 "../gnumach/mig/lexxer.l"
RETURN(syCString);
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 143 "../gnumach/mig/lexxer.l"
FRETURN(flLong);
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 144 "../gnumach/mig/lexxer.l"
FRETURN(flNotLong);
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 145 "../gnumach/mig/lexxer.l"
FRETURN(flDealloc);
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 146 "../gnumach/mig/lexxer.l"
FRETURN(flNotDealloc);
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 147 "../gnumach/mig/lexxer.l"
FRETURN(flServerCopy);
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 148 "../gnumach/mig/lexxer.l"
FRETURN(flCountInOut);
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 150 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_POLYMORPHIC,word_size_in_bits);
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 152 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_UNSTRUCTURED,0);
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 153 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_BIT,1);
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 154 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_BOOLEAN,32);
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 155 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_INTEGER_16,16);
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 156 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_INTEGER_32,32);
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 157 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_INTEGER_64,64);
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 158 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_CHAR,8);
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 159 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_BYTE,8);
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 160 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_INTEGER_8,8);
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 161 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_REAL,0);
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 162 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_STRING,0);
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 163 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_STRING_C,0);
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 165 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_MOVE_RECEIVE,MACH_MSG_TYPE_PORT_RECEIVE,word_size_in_bits);
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 166 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_COPY_SEND,MACH_MSG_TYPE_PORT_SEND,word_size_in_bits);
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 167 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_MAKE_SEND,MACH_MSG_TYPE_PORT_SEND,word_size_in_bits);
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 168 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_MOVE_SEND,MACH_MSG_TYPE_PORT_SEND,word_size_in_bits);
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 169 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_MAKE_SEND_ONCE,MACH_MSG_TYPE_PORT_SEND_ONCE,word_size_in_bits);
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 170 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_MOVE_SEND_ONCE,MACH_MSG_TYPE_PORT_SEND_ONCE,word_size_in_bits);
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 172 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_PORT_NAME,word_size_in_bits);
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 173 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_POLYMORPHIC,MACH_MSG_TYPE_PORT_RECEIVE,word_size_in_bits);
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 174 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_POLYMORPHIC,MACH_MSG_TYPE_PORT_SEND,word_size_in_bits);
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 175 "../gnumach/mig/lexxer.l"
TPRETURN(MACH_MSG_TYPE_POLYMORPHIC,MACH_MSG_TYPE_PORT_SEND_ONCE,word_size_in_bits);
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 176 "../gnumach/mig/lexxer.l"
TRETURN(MACH_MSG_TYPE_POLYMORPHIC,0);
	YY_BREAK
case 70:
YY_RULE_SETUP
#line 178 "../gnumach/mig/lexxer.l"
RETURN(syColon);
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 179 "../gnumach/mig/lexxer.l"
RETURN(sySemi);
	YY_BREAK
case 72:
YY_RULE_SETUP
#line 180 "../gnumach/mig/lexxer.l"
RETURN(syComma);
	YY_BREAK
case 73:
YY_RULE_SETUP
#line 181 "../gnumach/mig/lexxer.l"
RETURN(syPlus);
	YY_BREAK
case 74:
YY_RULE_SETUP
#line 182 "../gnumach/mig/lexxer.l"
RETURN(syMinus);
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 183 "../gnumach/mig/lexxer.l"
RETURN(syStar);
	YY_BREAK
case 76:
YY_RULE_SETUP
#line 184 "../gnumach/mig/lexxer.l"
RETURN(syDiv);
	YY_BREAK
case 77:
YY_RULE_SETUP
#line 185 "../gnumach/mig/lexxer.l"
RETURN(syLParen);
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 186 "../gnumach/mig/lexxer.l"
RETURN(syRParen);
	YY_BREAK
case 79:
YY_RULE_SETUP
#line 187 "../gnumach/mig/lexxer.l"
RETURN(syEqual);
	YY_BREAK
case 80:
YY_RULE_SETUP
#line 188 "../gnumach/mig/lexxer.l"
RETURN(syCaret);
	YY_BREAK
case 81:
YY_RULE_SETUP
#line 189 "../gnumach/mig/lexxer.l"
RETURN(syTilde);
	YY_BREAK
case 82:
YY_RULE_SETUP
#line 190 "../gnumach/mig/lexxer.l"
RETURN(syLAngle);
	YY_BREAK
case 83:
YY_RULE_SETUP
#line 191 "../gnumach/mig/lexxer.l"
RETURN(syRAngle);
	YY_BREAK
case 84:
YY_RULE_SETUP
#line 192 "../gnumach/mig/lexxer.l"
RETURN(syLBrack);
	YY_BREAK
case 85:
YY_RULE_SETUP
#line 193 "../gnumach/mig/lexxer.l"
RETURN(syRBrack);
	YY_BREAK
case 86:
YY_RULE_SETUP
#line 194 "../gnumach/mig/lexxer.l"
RETURN(syBar);
	YY_BREAK
case 87:
YY_RULE_SETUP
#line 196 "../gnumach/mig/lexxer.l"
{ yylval.identifier = strmake(yytext);
			  RETURN(syIdentifier); }
	YY_BREAK
case 88:
YY_RULE_SETUP
#line 198 "../gnumach/mig/lexxer.l"
{ yylval.number = atoi(yytext); RETURN(syNumber); }
	YY_BREAK
case 89:
YY_RULE_SETUP
#line 200 "../gnumach/mig/lexxer.l"
{ yylval.string = strmake(yytext);
			  BEGIN Normal; RETURN(syString); }
	YY_BREAK
case 90:
YY_RULE_SETUP
#line 202 "../gnumach/mig/lexxer.l"
{ yylval.string = strmake(yytext);
			  BEGIN Normal; RETURN(syFileName); }
	YY_BREAK
case 91:
YY_RULE_SETUP
#line 204 "../gnumach/mig/lexxer.l"
{ yylval.string = strmake(yytext);
			  BEGIN Normal; RETURN(syQString); }
	YY_BREAK
case 92:
YY_RULE_SETUP
#line 207 "../gnumach/mig/lexxer.l"
{ doSharp(yytext+1);
					  SAVE_BEGIN;
					  BEGIN SkipToEOL; }
	YY_BREAK
case 93:
YY_RULE_SETUP
#line 210 "../gnumach/mig/lexxer.l"
{ doSharp(yytext+1);
					  SAVE_BEGIN;
					  BEGIN SkipToEOL; }
	YY_BREAK
case 94:
YY_RULE_SETUP
#line 213 "../gnumach/mig/lexxer.l"
{ yyerror("illegal # directive");
					  SAVE_BEGIN;
					  BEGIN SkipToEOL; }
	YY_BREAK
case 95:
YY_RULE_SETUP
#line 217 "../gnumach/mig/lexxer.l"
RSTR_BEGIN;
	YY_BREAK
case 96:
YY_RULE_SETUP
#line 218 "../gnumach/mig/lexxer.l"
;
	YY_BREAK
case 97:
YY_RULE_SETUP
#line 220 "../gnumach/mig/lexxer.l"
;
	YY_BREAK
case 98:
YY_RULE_SETUP
#line 221 "../gnumach/mig/lexxer.l"
{ lineno++; }
	YY_BREAK
case 99:
YY_RULE_SETUP
#line 222 "../gnumach/mig/lexxer.l"
{ BEGIN Normal; RETURN(syError); }
	YY_BREAK
case 100:
YY_RULE_SETUP
#line 224 "../gnumach/mig/lexxer.l"
ECHO;
	YY_BREAK
#line 1606 "lex.yy.c"
case YY_STATE_EOF(INITIAL):
case YY_STATE_EOF(Normal):
case YY_STATE_EOF(String):
case YY_STATE_EOF(FileName):
case YY_STATE_EOF(QString):
case YY_STATE_EOF(SkipToEOL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a singled characater, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yy_c_buf_p - yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset =
				(int) (yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yy_flex_realloc( (void *) b->yy_ch_buf,
							 b->yy_buf_size + 2 );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yytext_ptr = &yy_current_buffer->yy_ch_buf[0];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 515 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
#endif
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 515 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 514);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifndef YY_NO_UNPUT
#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp )
#else
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
#endif
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;


	yytext_ptr = yy_bp;
	yy_hold_char = *yy_cp;
	yy_c_buf_p = yy_cp;
	}
#endif	/* ifndef YY_NO_UNPUT */


#ifdef __cplusplus
static int yyinput()
#else
static int input()
#endif
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			yytext_ptr = yy_c_buf_p;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						{
						yy_c_buf_p =
						yytext_ptr + YY_MORE_ADJ;
						return EOF;
						}

					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;
					break;

				case EOB_ACT_LAST_MATCH:
#ifdef __cplusplus
					YY_FATAL_ERROR(
					"unexpected last match in yyinput()" );
#else
					YY_FATAL_ERROR(
					"unexpected last match in input()" );
#endif
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;

	yy_current_buffer->yy_at_bol = (c == '\n');

	return c;
	}


#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file )
#else
void yyrestart( input_file )
FILE *input_file;
#endif
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void yy_load_buffer_state( void )
#else
void yy_load_buffer_state()
#endif
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b )
#else
void yy_delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( ! b )
		return;

	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (void *) b->yy_ch_buf );

	yy_flex_free( (void *) b );
	}


#ifndef YY_ALWAYS_INTERACTIVE
#ifndef YY_NEVER_INTERACTIVE
extern int isatty YY_PROTO(( int ));
#endif
#endif

#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif


	{
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

#if YY_ALWAYS_INTERACTIVE
	b->yy_is_interactive = 1;
#else
#if YY_NEVER_INTERACTIVE
	b->yy_is_interactive = 0;
#else
	b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
	}


#ifdef YY_USE_PROTOS
void yy_flush_buffer( YY_BUFFER_STATE b )
#else
void yy_flush_buffer( b )
YY_BUFFER_STATE b;
#endif

	{
	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == yy_current_buffer )
		yy_load_buffer_state();
	}


#ifndef YY_NO_SCAN_BUFFER
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_buffer( char *base, yy_size_t size )
#else
YY_BUFFER_STATE yy_scan_buffer( base, size )
char *base;
yy_size_t size;
#endif
	{
	YY_BUFFER_STATE b;

	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b );

	return b;
	}
#endif


#ifndef YY_NO_SCAN_STRING
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_string( yyconst char *str )
#else
YY_BUFFER_STATE yy_scan_string( str )
yyconst char *str;
#endif
	{
	int len;
	for ( len = 0; str[len]; ++len )
		;

	return yy_scan_bytes( str, len );
	}
#endif


#ifndef YY_NO_SCAN_BYTES
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_bytes( yyconst char *bytes, int len )
#else
YY_BUFFER_STATE yy_scan_bytes( bytes, len )
yyconst char *bytes;
int len;
#endif
	{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;

	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) yy_flex_alloc( n );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
	}
#endif


#ifndef YY_NO_PUSH_STATE
#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state )
#else
static void yy_push_state( new_state )
int new_state;
#endif
	{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
		{
		yy_size_t new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yy_flex_realloc(
					(void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}
#endif


#ifndef YY_NO_POP_STATE
static void yy_pop_state()
	{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
	}
#endif


#ifndef YY_NO_TOP_STATE
static int yy_top_state()
	{
	return yy_start_stack[yy_start_stack_ptr - 1];
	}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

#ifdef YY_USE_PROTOS
static void yy_fatal_error( yyconst char msg[] )
#else
static void yy_fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n - YY_MORE_ADJ; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, yyconst char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
yyconst char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( yy_size_t size )
#else
static void *yy_flex_alloc( size )
yy_size_t size;
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, yy_size_t size )
#else
static void *yy_flex_realloc( ptr, size )
void *ptr;
yy_size_t size;
#endif
	{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}

#if YY_MAIN
int main()
	{
	yylex();
	return 0;
	}
#endif
#line 224 "../gnumach/mig/lexxer.l"


extern void
LookNormal(void)
{
    if (inname == 0)
    {
	inname = strmake("(unknown)");
	lineno = 0;
    }

    BEGIN Normal;
}

extern void
LookString(void)
{
    BEGIN String;
}

extern void
LookQString(void)
{
    BEGIN QString;
}

extern void
LookFileName(void)
{
    BEGIN FileName;
}

static void
doSharp(const char *body)
{
    register const char *startName;

    lineno = atoi(body);
    startName = strchr(body, '"');
    if (startName != NULL)
    {
	*strrchr(body, '"') = '\0';
	strfree(inname);
	inname = strmake(startName+1);
    }
}
