#define UTS_RELEASE "1.3.68"
#define UTS_VERSION "#1 Thu Feb 29 16:37:10 MST 1996"
#define LINUX_COMPILE_TIME "09:03:52"
#define LINUX_COMPILE_BY "goel"
#define LINUX_COMPILE_HOST "stamp.cs.utah.edu"
#define LINUX_COMPILE_DOMAIN "cs.utah.edu"
#define LINUX_COMPILER "gcc version 2.7.2"
#define LINUX_VERSION_CODE (65536 + 4 * 256)
